from textwrap import dedent

from .frontend_config_handler import FrontendConfigHandler


class MockFrontendConfigHandler(FrontendConfigHandler):
    @property
    def _config_str(self):
        return dedent('''
        cat:
            someConfigKey: lel
            otherStuff:    42
        foo:
            hey:           yaaay
        ''')


class MockConnector:
    def __init__(self):
        self.messages = []

    def send_message(self, message, **_):
        self.messages.append(message)


def test_frontend_config_handler():
    connector = MockConnector()
    handler = MockFrontendConfigHandler('')

    handler.handle_event({'key': 'frontend.ready'}, connector)
    assert connector.messages[0] == {
        'key': 'frontend.cat',
        'someConfigKey': 'lel',
        'otherStuff': 42
    }
    assert connector.messages[1] == {
        'key': 'frontend.foo',
        'hey': 'yaaay'
    }
    assert connector.messages[-1] == {'key': 'frontend.ready'}
