from tfw.internals.networking import Scope

from .message_storage import FrontendMessageStorage


class FrontendProxyHandler:
    keys = ['console', 'dashboard', 'frontend', 'message', 'ide.read', 'deploy.finish']
    type_id = 'ControlEventHandler'

    def __init__(self):
        self.connector = None
        self._frontend_message_storage = FrontendMessageStorage()

    def send_message(self, message):
        self.connector.send_message(message, scope=Scope.WEBSOCKET)

    def handle_event(self, message, _):
        self._frontend_message_storage.save_message(message)
        if message['key'] == 'frontend.ready':
            self.recover_frontend()
            return
        if self._filter_message(message):
            self.send_message(message)

    @staticmethod
    def _filter_message(message):
        return not message['key'].startswith((
            'ide',
            'message.queue'
        ))

    def recover_frontend(self):
        for message in self._frontend_message_storage.messages:
            self.send_message(message)
