import logging

from tfw.internals.crypto import KeyManager, sign_message
from tfw.internals.networking import Scope, Intent

from .fsm_updater import FSMUpdater

LOG = logging.getLogger(__name__)


class FSMHandler:
    keys = ['fsm']
    type_id = 'ControlEventHandler'

    def __init__(self, *, fsm_type):
        self.connector = None
        self.fsm = fsm_type()
        self._fsm_updater = FSMUpdater(self.fsm)
        self.auth_key = KeyManager().auth_key

        self.command_handlers = {
            'fsm.trigger': self.handle_trigger,
            'fsm.update' : self.handle_update
        }

    def handle_event(self, message, _):
        try:
            message = self.command_handlers[message['key']](message)
            if message:
                fsm_update_message = self._fsm_updater.fsm_update
                sign_message(self.auth_key, fsm_update_message)
                self.connector.send_message(fsm_update_message, Scope.BROADCAST, Intent.EVENT)
        except KeyError:
            LOG.error('IGNORING MESSAGE: Invalid message received: %s', message)

    def handle_trigger(self, message): # pylint: disable=inconsistent-return-statements
        if self.fsm.step(message['transition']):
            return message

    def handle_update(self, message):
        # pylint: disable=no-self-use
        return message
