from tfw.internals.networking import Scope, Intent
from tfw.internals.inotify import InotifyObserver

from .supervisor import ProcessLogManager


class LogInotifyObserver(InotifyObserver, ProcessLogManager):
    def __init__(self, connector, process_name, supervisor_uri, log_tail=0):
        self._connector = connector
        self._process_name = process_name
        self.log_tail = log_tail
        self._procinfo = None
        ProcessLogManager.__init__(self, supervisor_uri)
        InotifyObserver.__init__(self, self._get_logfiles())

    def _get_logfiles(self):
        self._procinfo = self.supervisor.getProcessInfo(self._process_name)
        return self._procinfo['stdout_logfile'], self._procinfo['stderr_logfile']

    @property
    def process_name(self):
        return self._process_name

    @process_name.setter
    def process_name(self, process_name):
        self._process_name = process_name
        self.paths = self._get_logfiles()

    def on_modified(self, event):
        self._connector.send_message({
            'key': 'process.log.new',
            'stdout': self.read_stdout(self.process_name, tail=self.log_tail),
            'stderr': self.read_stderr(self.process_name, tail=self.log_tail)
        }, Scope.BROADCAST, Intent.EVENT)
