from abc import ABCMeta
from contextlib import suppress


class TypeIdRegistryMeta(ABCMeta):
    def __init__(cls, name, bases, attrs):
        with suppress(AttributeError):
            if cls.__name__ in cls._type_id_registry:
                raise RuntimeError('Type id must be unique!')
            cls._type_id_registry[cls.__name__] = cls
        super().__init__(name, bases, attrs)


class TypeIdRegistryMixin(metaclass=TypeIdRegistryMeta):
    @classmethod
    def get_type(cls, type_id):
        try:
            return cls._type_id_registry[type_id]
        except KeyError:
            raise RuntimeError(f'No type with id {type_id}!')

    @classmethod
    def build_type(cls, type_id, *args, **kwargs):
        return cls.get_type(type_id)(*args, **kwargs)
